@@ -22,12 +22,10 @@
 
 #include "webarchiverdebug.h"
 
-
-K_PLUGIN_FACTORY(PluginWebArchiverFactory, registerPlugin<PluginWebArchiver>();)
-
+K_PLUGIN_CLASS_WITH_JSON(PluginWebArchiver, "plugin_webarchiver.json")
 
 PluginWebArchiver::PluginWebArchiver(QObject *parent, const QVariantList &args)
-    : Plugin(parent)
+    : KonqParts::Plugin(parent)
 {
     QAction *a = actionCollection()->addAction(QStringLiteral("archivepage"));
     a->setText(i18n("Archive Web Page..."));
